@@ -42,12 +42,19 @@
 #include "pycairo.h"
 #include "compat.h"
 
+int Pycairo_fspath_converter (PyObject *obj, char** result);
+int Pycairo_fspath_none_converter (PyObject *obj, char** result);
+int Pycairo_writer_converter (PyObject *obj, PyObject** file);
+int Pycairo_reader_converter (PyObject *obj, PyObject** file);
+int Pycairo_is_fspath (PyObject *obj);
 
 cairo_glyph_t * _PycairoGlyphs_AsGlyphs (PyObject *py_object, int *num_glyphs);
 int _PyGlyph_AsGlyph (PyObject *pyobj, cairo_glyph_t *glyph);
 int _PyTextCluster_AsTextCluster (PyObject *pyobj,
                                   cairo_text_cluster_t *cluster);
 
+int _conv_pyobject_to_ulong (PyObject *pyobj, unsigned long *result);
+
 PyObject *_Pycairo_Get_Error(void);
 
 PyObject* Pycairo_richcompare (void* a, void *b, int op);
